@@ -26,6 +26,8 @@
 #include	<stdio.h>
 #include	<time.h>
 
+/* for struct in6_addr */
+#include	<netinet/in.h>
 
 /* for struct addrinfo and sockaddr_storage */
 #include <sys/socket.h>
@@ -44,17 +46,18 @@ extern "C" {
  * @{
  */
 
-#define AUTH_PASS_LEN		(3 * 16) /* multiple of 16 */
+#define AUTH_PASS_LEN		(7 * 16) /* multiple of 16 */
 #define AUTH_ID_LEN		64
 
-#define BUFFER_LEN		8192
+#define RC_BUFFER_LEN		8192
 
-#define NAME_LENGTH		32
+#define RC_NAME_LENGTH		32
 
 #define MAX_SECRET_LENGTH	(6 * 16) /* MUST be multiple of 16 */
 
-#define VENDOR(x)		(((x) >> 16) & 0xffff)
-#define ATTRID(x)		((x) & 0xffff)
+#define VENDOR_BIT_SIZE		32
+#define VENDOR(x)		(((x) >> VENDOR_BIT_SIZE) & 0xffffffff)
+#define ATTRID(x)		((x) & 0xffffffff)
 
 #define PW_MAX_MSG_SIZE		4096
 
@@ -67,7 +70,7 @@ typedef enum rc_type {
 
 /* defines for config.c */
 
-#define SERVER_MAX 8
+#define RC_SERVER_MAX 8
 
 #define AUTH_LOCAL_FST	(1<<0)
 #define AUTH_RADIUS_FST	(1<<1)
@@ -77,15 +80,16 @@ typedef enum rc_type {
 struct rc_conf;
 typedef struct rc_conf rc_handle;
 
-/** \struct server Avoid using this structure directly, it is included for backwards compatibility only.
+/** \struct server
+ * Avoid using this structure directly, it is included for backwards compatibility only.
  * Several of its fields have been deprecated.
  */
 typedef struct server {
 	int   max;
-	char *name[SERVER_MAX];
-	uint16_t port[SERVER_MAX];
-	char *secret[SERVER_MAX];
-	double deadtime_ends[SERVER_MAX]; //!< unused
+	char *name[RC_SERVER_MAX];
+	uint16_t port[RC_SERVER_MAX];
+	char *secret[RC_SERVER_MAX];
+	double deadtime_ends[RC_SERVER_MAX]; //!< unused
 } SERVER;
 
 /** \enum rc_socket_type Indicate the type of the socket
@@ -111,7 +115,8 @@ typedef enum rc_attr_type {
 	PW_TYPE_IPADDR=2,	//!< The attribute is an IPv4 address in host-byte order.
 	PW_TYPE_DATE=3,		//!< The attribute contains a 32-bit number indicating the seconds since epoch.
 	PW_TYPE_IPV6ADDR=4,	//!< The attribute is an 128-bit IPv6 address.
-	PW_TYPE_IPV6PREFIX=5    //!< The attribute is an IPv6 prefix; the lvalue will indicate its size.
+	PW_TYPE_IPV6PREFIX=5,   //!< The attribute is an IPv6 prefix; the lvalue will indicate its size.
+	PW_TYPE_MAX=6   	//!< Maximum number of types (last+1)
 } rc_attr_type;
 
 /** \enum rc_standard_codes Standard RADIUS request codes
@@ -424,24 +429,24 @@ enum rc_vendor_attr_roaringpenguin {
 
 typedef struct dict_attr
 {
-	char              name[NAME_LENGTH + 1];	//!< attribute name.
-	int               value;			//!< attribute index.
+	char              name[RC_NAME_LENGTH + 1];	//!< attribute name.
+	uint64_t          value;			//!< attribute index and vendor number; use VENDOR() and ATTRID() to separate.
 	rc_attr_type      type;				//!< string, int, etc..
 	struct dict_attr *next;
 } DICT_ATTR;
 
 typedef struct dict_value
 {
-	char               attrname[NAME_LENGTH +1];
-	char               name[NAME_LENGTH + 1];
-	int                value;
+	char               attrname[RC_NAME_LENGTH +1];
+	char               name[RC_NAME_LENGTH + 1];
+	uint32_t           value;
 	struct dict_value *next;
 } DICT_VALUE;
 
 typedef struct dict_vendor
 {
-	char               vendorname[NAME_LENGTH +1];
-	int                vendorpec;
+	char               vendorname[RC_NAME_LENGTH +1];
+	uint32_t           vendorpec;
 	struct dict_vendor *next;
 } DICT_VENDOR;
 
@@ -451,6 +456,7 @@ typedef struct dict_vendor
 /** \enum rc_send_status Return codes for rc_send_server()
  */
 typedef enum rc_send_status {
+	NETUNREACH_RC=-4,
 	BADRESPID_RC=-3,
 	BADRESP_RC=-2,
 	ERROR_RC=-1,
@@ -463,12 +469,13 @@ typedef enum rc_send_status {
 
 # define AUTH_STRING_LEN		253	 /* maximum of 253 */
 
-/** \struct rc_value_pair Avoid using this structure directly. Use the rc_avpair_get_ functions.
+/** \struct rc_value_pair
+ * Avoid using this structure directly. Use the rc_avpair_get_ functions.
  */
 typedef struct rc_value_pair
 {
-	char               name[NAME_LENGTH + 1];	//!< attribute name if known.
-	unsigned           attribute;			//!< attribute numeric value of type rc_attr_id.
+	char               name[RC_NAME_LENGTH + 1];	//!< attribute name if known.
+	uint64_t           attribute;			//!< attribute numeric value of type rc_attr_id including vendor; use VENDOR() and ATTRID() to separate.
 	rc_attr_type	   type;			//!< attribute type.
 	uint32_t           lvalue;			//!< attribute value if type is PW_TYPE_INTEGER, PW_TYPE_DATE or PW_TYPE_IPADDR.
 	char               strvalue[AUTH_STRING_LEN + 1]; //!< contains attribute value in other cases.
@@ -494,11 +501,11 @@ typedef struct send_data /* Used to pass information to sendserver() function */
 struct rc_aaa_ctx_st;
 typedef struct rc_aaa_ctx_st RC_AAA_CTX;
 
-#ifndef MIN
-#define MIN(a, b)     ((a) < (b) ? (a) : (b))
+#ifndef RC_MIN
+#define RC_MIN(a, b)     ((a) < (b) ? (a) : (b))
 #endif
-#ifndef MAX
-#define MAX(a, b)     ((a) > (b) ? (a) : (b))
+#ifndef RC_MAX
+#define RC_MAX(a, b)     ((a) > (b) ? (a) : (b))
 #endif
 
 #ifndef PATH_MAX
@@ -531,7 +538,7 @@ typedef struct rc_aaa_ctx_st RC_AAA_CTX;
  * parsing the file, simplifying application configuration and administration.
  *
  * Alternative operation without a configuration file is also possible, see
- * rc_add_config().
+ * rc_add_config() and rc_apply_config().
  *
  * Check radexample.c for a functional example.
  *
@@ -559,13 +566,13 @@ typedef struct rc_aaa_ctx_st RC_AAA_CTX;
 
 /* avpair.c */
 
-VALUE_PAIR *rc_avpair_add (rc_handle const *rh, VALUE_PAIR **list, int attrid, void const *pval, int len, int vendorpec);
+VALUE_PAIR *rc_avpair_add (rc_handle const *rh, VALUE_PAIR **list, uint32_t attrid, void const *pval, int len, uint32_t vendorspec);
 int rc_avpair_assign (VALUE_PAIR *vp, void const *pval, int len);
-VALUE_PAIR *rc_avpair_new (rc_handle const *rh, int attrid, void const *pval, int len, int vendorpec);
+VALUE_PAIR *rc_avpair_new (rc_handle const *rh, uint32_t attrid, void const *pval, int len, uint32_t vendorspec);
 VALUE_PAIR *rc_avpair_gen(rc_handle const *rh, VALUE_PAIR *pair, unsigned char const *ptr,
-			  int length, int vendorpec);
-void rc_avpair_remove (VALUE_PAIR **list, int attrid, int vendorpec);
-VALUE_PAIR *rc_avpair_get (VALUE_PAIR *vp, int attrid, int vendorpec);
+			  int length, uint32_t vendorspec);
+void rc_avpair_remove (VALUE_PAIR **list, uint32_t attrid, uint32_t vendorspec);
+VALUE_PAIR *rc_avpair_get (VALUE_PAIR *vp, uint32_t attrid, uint32_t vendorspec);
 VALUE_PAIR *rc_avpair_copy(VALUE_PAIR *p);
 void rc_avpair_insert(VALUE_PAIR **a, VALUE_PAIR *p, VALUE_PAIR *b);
 void rc_avpair_free (VALUE_PAIR *pair);
@@ -609,6 +616,7 @@ char *rc_conf_str(rc_handle const *rh, char const *optname);
 int rc_conf_int(rc_handle const *rh, char const *optname);
 SERVER *rc_conf_srv(rc_handle const *rh, char const *optname);
 int rc_test_config(rc_handle *rh, char const *filename);
+int rc_apply_config(rc_handle *rh);
 int rc_find_server_addr (rc_handle const *rh, char const *server_name,
                          struct addrinfo** info, char *secret, rc_type type);
 void rc_config_free(rc_handle *rh);
@@ -621,11 +629,17 @@ rc_socket_type rc_get_socket_type(rc_handle * rh);
 /* dict.c */
 
 int rc_read_dictionary (rc_handle *rh, char const *filename);
-DICT_ATTR *rc_dict_getattr(rc_handle const *rh, int attribute);
+int rc_read_dictionary_from_buffer (rc_handle *rh, char const *buf, size_t size);
+
+DICT_ATTR *rc_dict_addattr(rc_handle *rh, char const * namestr, uint32_t value, int type, uint32_t vendorspec);
+DICT_VALUE *rc_dict_addval(rc_handle *rh, char const * attrstr, char const * namestr, uint32_t value);
+DICT_VENDOR *rc_dict_addvend(rc_handle *rh, char const * vendorname, uint32_t value);
+
+DICT_ATTR *rc_dict_getattr(rc_handle const *rh, uint64_t attribute);
 DICT_ATTR *rc_dict_findattr(rc_handle const *rh, char const *attrname);
 DICT_VALUE *rc_dict_findval(rc_handle const *rh, char const *valname);
 DICT_VENDOR *rc_dict_findvend(rc_handle const *rh, char const *vendorname);
-DICT_VENDOR *rc_dict_getvend (rc_handle const *rh, int vendorpec);
+DICT_VENDOR *rc_dict_getvend (rc_handle const *rh, uint32_t vendorspec);
 DICT_VALUE *rc_dict_getval(rc_handle const *rh, uint32_t value, char const *attrname);
 void rc_dict_free(rc_handle *rh);
 
